"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var classnames = require("classnames");
var data = require("../data");
var FormBuilderDroppable_1 = require("./FormBuilderDroppable");
var FormBuilderContext_1 = require("./FormBuilderContext");
var utils_1 = require("../utils");
// FormBuilder expects a list of field definitions and will wrap each field definition
// in utility components for editing, dragging, and dropping. The FormBuilder uses
// a registry to determine which class is responsible for rendering the field type.
var FormBuilder = (function (_super) {
    __extends(FormBuilder, _super);
    function FormBuilder(props) {
        var _this = _super.call(this, props) || this;
        _this.builderError = {};
        _this.renderField = _this.renderField.bind(_this);
        _this.onDrop = _this.onDrop.bind(_this);
        _this.onEditField = _this.onEditField.bind(_this);
        _this.onDeleteField = _this.onDeleteField.bind(_this);
        _this.onFieldChanged = _this.onFieldChanged.bind(_this);
        _this.onFieldError = _this.onFieldError.bind(_this);
        _this.canDrop = _this.canDrop.bind(_this);
        return _this;
    }
    // NOTE(andrews): This is public API. Consumers should call this method
    // directly if they wish to add a field to the fields array. This encapsulates
    // all of the behavior surrounding adding a field such as cloning the field,
    // ensuring an ID exists, and running hooks. Internally this method calls onDrop.
    FormBuilder.prototype.addField = function (field) {
        var targetIndex = this.props.fields.length;
        var target = {
            field: this.props.fields[targetIndex],
            index: targetIndex,
        };
        var source = {
            field: field,
            index: null,
        };
        this.onDrop(target, source, false);
    };
    // onEditField is called when the user wants to edit a field.
    // If the `onBeforeEditField` hook is provided, it will be used
    // to determine if the field can be edited.
    FormBuilder.prototype.onEditField = function (field) {
        var hook = this.props.onBeforeEditField;
        if (hook && !hook(field)) {
            return;
        }
        this.props.onFieldEditing(field);
    };
    // onDeleteField is called when the user wants to delete a field.
    // If the `onBeforeDeleteField` hook is provided, it will be used
    // to determine if the field can be deleted.
    FormBuilder.prototype.onDeleteField = function (index) {
        var field = this.props.fields[index];
        var hook = this.props.onBeforeDeleteField;
        if (hook && !hook(field)) {
            return;
        }
        var fields = this.props.fields.concat([]);
        fields.splice(index, 1);
        this.props.onChange(fields, {
            action: data.FieldAction.Delete,
            source: field
        });
        if (this.builderError[field.id]) {
            // Resets the deleted field error.
            this.onFieldError(field, index, null);
        }
    };
    // onDrop is called whenever a field is dropped on a <Droppable> component.
    // The field being dropped can come from one of two places:
    //      1. The FieldSelector component
    //      2. A <Draggable/> component being reordered.
    // target and source both contain the index and the IField of the field
    // being dragged, and the field being dropped on.
    // If a field is being dragged from <FieldSelector> then it will not have an index
    // and should simply be inserted.
    // If the source field comes from a <Draggable> component then it must
    // go through removal and insertion. Removal and insertion depend
    // on the source index to keep the integrity of the target index.
    FormBuilder.prototype.onDrop = function (target, source, didDrop) {
        if (didDrop) {
            return;
        }
        if (target.index === source.index) {
            return;
        }
        var sourceField = source.field;
        var isAdd = (source.index === null);
        if (isAdd) {
            // NOTE: If source is from the FieldSelector, we should create a clone field.
            sourceField = JSON.parse(JSON.stringify(sourceField));
            this.ensureID(sourceField);
            var hook = this.props.onBeforeAddField;
            if (hook && !hook(sourceField)) {
                return;
            }
        }
        else if (this.props.fields.indexOf(target.field) !== target.index &&
            this.props.fields.indexOf(source.field) !== source.index) {
            // NOTE: For re-ordering, make sure the target & source are in the same level.
            return;
        }
        var fields = this.props.fields.concat([]);
        if (isAdd) {
            fields.splice(target.index, 0, sourceField);
        }
        else if (source.index < target.index) {
            fields.splice(target.index, 0, sourceField);
            fields.splice(source.index, 1);
        }
        else {
            fields.splice(source.index, 1);
            fields.splice(target.index, 0, sourceField);
        }
        this.props.onChange(fields, {
            action: isAdd ? data.FieldAction.Add : data.FieldAction.Change,
            source: sourceField
        });
    };
    FormBuilder.prototype.ensureID = function (field) {
        if (!field.id) {
            var idFunc = this.props.idGenerator || utils_1.generateID;
            field.id = idFunc();
        }
    };
    FormBuilder.prototype.onFieldChanged = function (field, index, change) {
        var fields = this.props.fields.slice();
        fields[index] = field;
        this.props.onChange(fields, change);
    };
    FormBuilder.prototype.onFieldError = function (field, index, error) {
        if (error) {
            this.builderError[field.id] = error;
        }
        else {
            delete this.builderError[field.id];
        }
        var hasError = Object.keys(this.builderError).length > 0;
        this.props.onError(hasError ? this.builderError : null);
    };
    // NOTE(andrews) canDrop holds the business logic for determining if the source item can
    // be dropped on the target item.
    FormBuilder.prototype.canDrop = function (source, target) {
        // NOTE(andrews): Prohibit moving non-new fields between parents
        // (eg. moving non-nested field into nested form)
        if (source.field.id && source.parentId !== target.parentId) {
            return false;
        }
        // NOTE(andrews): Prohibit the field from being dropped on itself.
        // This prevents the indicator from showing up.
        if (target.field && source.field.id === target.field.id) {
            return false;
        }
        // NOTE(andrews): Prohibit the field from dropped back into the same index.
        if (source.index !== undefined && source.index === target.index - 1) {
            return false;
        }
        if (this.props.canDrop) {
            return this.props.canDrop(source, target);
        }
        return true;
    };
    // renderField takes the field.type to be rendered and looks up the
    // appropriate component class in the registry that can render the component.
    // The rendered component is passed the field as a prop.
    FormBuilder.prototype.renderField = function (field, index) {
        var fieldDef = this.props.registry[field.type];
        if (!fieldDef || !fieldDef.builder) {
            console.warn('Field defintion is not registered: ' + field.type);
            return;
        }
        var fieldBuilderProps = {
            field: field,
            index: index,
            registry: this.props.registry,
            editButton: this.props.editButton,
            deleteButton: this.props.deleteButton,
            editingFieldId: this.props.editingFieldId,
            fields: this.props.fields,
            onFieldEditing: this.props.onFieldEditing,
            onChange: this.onFieldChanged,
            onError: this.onFieldError,
            onBeforeAddField: this.props.onBeforeAddField,
            canDrag: this.props.canDrag,
            canDrop: this.canDrop,
            parentId: this.props.parentId,
            onDeleteField: this.onDeleteField,
            onDrop: this.onDrop,
            onEditField: this.onEditField,
            idGenerator: this.props.idGenerator,
        };
        var component = React.createElement(fieldDef.builder, fieldBuilderProps);
        return (React.createElement("div", { className: 'form-builder-field', key: field.id }, component));
    };
    // render displays a list of fields that have been wrapped in components for
    // providing editing, dragging, and dropping functionality. At the end of the list
    // is a <Droppable> field.
    FormBuilder.prototype.render = function () {
        var isEmpty = this.props.fields.length === 0;
        var emptyLayout = this.props.emptyLayout;
        return (React.createElement(FormBuilderContext_1.FormBuilderContext, null,
            React.createElement("div", { className: classnames('form-builder', { 'form-builder-empty': isEmpty }) },
                this.props.fields.map(this.renderField),
                React.createElement(FormBuilderDroppable_1.FormBuilderDroppable, { parentId: this.props.parentId, canDrop: this.canDrop, index: this.props.fields.length, field: null, onDrop: this.onDrop },
                    React.createElement("div", { className: 'form-builder-default-droppable' }, isEmpty && emptyLayout)))));
    };
    return FormBuilder;
}(React.Component));
exports.FormBuilder = FormBuilder;
